/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.SwingUtil;

/**
 * A very simple Dialog with a message, a yes and no button.
 * The difference with a Messagebox is that this question box delivers
 * a yes / no result. A message box only has an OK button is not used for
 * user interaction, but user notification. The question box is used
 * for user interaction
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class QuestionBox extends JDialog {

    private JTextArea textMessage;
    private JButton buttonYes;
    private JButton buttonNo;
    private JPanel panel = new JPanel(false);

    private final JLabel labelIcon = ComponentFactory.getLabel("");

    public static Boolean result = null;

    public QuestionBox(String message) {
        super(SwingUtil.getRootFrame());     
        init(message);       
    }

    /**
     * Initializes and builds the dialog, then shows it. Modal is used.
     * @param message display String
     * @param parent the parent frame
     */
    public QuestionBox(String message, JFrame parent) {
        super(parent);
        init(message);
    }
    
    /**
     * Closes and disposes this dialog
     */
    public void close() {
        this.dispose();
    }    

    private void init(String message) {
        result = null;
        buildDialog();
        textMessage.setText(message);

        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

        this.pack();
        this.setModal(true);
        
        if (DataCrow.mainFrame != null) {
            Dimension size = DataCrow.mainFrame.getSize();
            Point location = DataCrow.mainFrame.getLocation();
            int x = ((location.x + size.width - getSize().width) / 2);
            int y = ((location.y + size.height  - getSize().height) / 2);        
            setLocation(x, y);
        } else {
        	setLocation(150, 150);
        }
        
        buttonYes.requestFocus();
        this.setVisible(true);
    }
    
    /**
     * Builds the dialog
     */
    private void buildDialog() {
        this.setResizable(false);
        this.getContentPane().setLayout(new GridLayout());

        // Input panel
        Color color = Settings.getValueAsColor(Repository.settings.stColor);
        this.setBackground(color);
        panel.setBackground(color);        
        panel.setLayout(Layout.getGBL());

        textMessage = ComponentFactory.getTextArea();
        textMessage.setBackground(getBackground());
        textMessage.setEditable(false);
        JScrollPane scrollIn = new JScrollPane(textMessage);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollIn.setPreferredSize(new Dimension(350,50));
        scrollIn.setBorder(null);
        
        buttonYes   = ComponentFactory.getButton(Resources.getText(DCLabels.lblYes), 100);
        buttonNo    = ComponentFactory.getButton(Resources.getText(DCLabels.lblNo),  100);
        
        buttonYes.addActionListener(new YesButtonAction());
        buttonNo.addActionListener(new NoButtonAction());
        
        labelIcon.setIcon(IconLibrary._icoQuestion);

        panel.setLayout(Layout.getGBL());
        panel.add(labelIcon,  Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0));            
        panel.add(scrollIn,   Layout.getGBC( 1, 0, 6, 3, 90.0, 90.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                              new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonYes, Layout.getGBC(  0, 3, 2, 1, 0.0, 0.0
                             ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0)); 
        panel.add(buttonNo,  Layout.getGBC(  2, 3, 1, 1, 0.0, 0.0
                             ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0)); 
            
        this.pack(); 
        this.setModal(true);
        this.setSize(Settings.getValueAsDimension(Repository.settings.stFieldSelectorSize));
        this.getContentPane().add(panel);
    }

    private class YesButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = new Boolean(true);
            close();
        }
    }

    private class NoButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            result = new Boolean(false);
            close();
        }
    }
}