/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Vector;

import net.sf.dc.core.objects.DCObject;

/**
 * This is the where part of the Query, it contains column names and their
 * values
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class QueryCondition {

    private String[][] columns;
    private ArrayList alValues = new ArrayList();
    private String reference = "";
    public String linkContion = "";
    
    /**
     * Creates a query condition
     * @param parent parent data crow object
     * @param select indicates whether this is a select query or not
     */
    public QueryCondition(DCObject parent, boolean select) {
        Vector v = new Vector();
        v.addElement(parent);
        
        // In the case of children, create a condition for both parent and child
        // in one query
        if (parent.getChildren().size() > 0 && select) {
            Object[] children = parent.getChildren().toArray();
            DCObject child = (DCObject) children[0];
            
            int length = parent.getChangedFields().length + 
                         child.getChangedFields().length;
            columns = new String[length][2];
            v.addElement(children[0]);
        } else {
            columns = new String[parent.getChangedFields().length][2];
        }
        
        for (Enumeration enum = v.elements(); enum.hasMoreElements(); ) {
        	DCObject dco = (DCObject) enum.nextElement();
        	Object[] values = dco.getChangedValues();
        	String[] fields = dco.getChangedFields();
        
            if (values.length > 0) {
                for (int i = 0; i < values.length; i++) {
                    Object value = values[i];
    
                    columns[i][0] = fields[i];
                    columns[i][1] = dco.getTableName();
    
                    if (value instanceof String) {
                        alValues.add("'" + value + "'");
                    } else if (value instanceof Integer) {
                        int integer = ((Integer) values[i]).intValue();
                        alValues.add(String.valueOf(integer));
                    } else if (value instanceof DCObject) {
                        alValues.add(((DCObject) value).getID());
                    } else {
                        alValues.add(value);
                    }
                }
            } else if (fields.length > 0){
                for (int i = 0; i < columns.length; i++) {
                    columns[i][0] = fields[i];
                    columns[i][1] = dco.getTableName();
                }
            }
        }
    }
    
    /**
     * Returns the columns
     */
    public String[][] getColumns() {
        return columns;
    }

    /**
     * Returns the values
     */
    public Collection getValues() {
        return alValues;
    }
}
