package net.sf.ext.mediamanager;

/**
 * @(#)FilePropertiesOGM.java 1.0 08.06.05 (dd.mm.yy)
 * 
 * Copyright (2003) Mediterranean
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License aloSng with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Boston, MA 02111.
 * 
 * Contact: mediterranean@users.sourceforge.net
 */

import java.io.FileInputStream;
import java.io.RandomAccessFile;
import java.util.StringTokenizer;

import net.sf.dc.messages.Messages;

class FilePropertiesOGM extends FileProperties {

    private final int Ogm = 0x4f676753;
    private final int End = 0x0373;
    private final int vide = 0x65646976;
    private final int audi = 0x69647561;
    private final int vorb = 0x62726f76;
    private final int text = 0x74786574;
    private final int OggS = 0x5367674f;
    private final int OggS1 = 0x4f676753;
    private final int ffff = 0xffffffff; //()

    private boolean quit = false;

    private float frameRate;

    /**
     * Processes a file from the given RandomAccessFile.
     */
    protected void process(RandomAccessFile dataStream) throws Exception {
        dataStream.seek(0);

        getInfo(dataStream);
        setDuration(getDuration(dataStream));

        calculateVideoBitRate(dataStream.length());
        setContainer("OGM");
    }

    private void getInfo(RandomAccessFile dataStream) throws Exception {

        int videoFccHandler;
        int audioFccHandler;
        int type;

        int twoByteCheck;

        String audioCodecs = "";
        int audioSampleRate = 0;
        int audioBitRate = 0;
        int audioChannels = 0;
        int safety = 0;
        boolean quit = false;
        int streamCounter = 0; // Counts audio and subtitle streams 

        while (!quit && safety++ < 15000) {

            if (       readUnsignedByte(dataStream) == 0x4f
                    && readUnsignedByte(dataStream) == 0x67
                    && readUnsignedByte(dataStream) == 0x67
                    && readUnsignedByte(dataStream) == 0x53) {

                skipBytes(dataStream, 1);

                if (readUnsignedByte(dataStream) == 2) {

                    long val = readUnsignedInt32(dataStream);
                    skipBytes(dataStream, 4);
                    int streamType = readUnsignedByte(dataStream);

                    skipBytes(dataStream, 12);

                    twoByteCheck = readUnsignedInt16(dataStream);
                    type = readUnsignedInt32(dataStream);

                    if (twoByteCheck == End) {
                        break;
                    }

                    // Video info 
                    if (type == vide) {
                        skipBytes(dataStream, 4);
                        videoFccHandler = readUnsignedInt32(dataStream);

                        FileInputStream fis = new FileInputStream("resources/FOURCCvideo.txt");
                        String videoCodec = findName(
                                fis, fromByteToAscii(videoFccHandler, 4));

                        setVideoCodec(videoCodec);

                        skipBytes(dataStream, 4);

                        frameRate = 
                            ((float) 10000000 / (float) readUnsignedInt32(dataStream)) * 1000;
                        frameRate = 
                            (float) ((double) (Math.ceil((double) frameRate)) / 1000);

                        setVideoRate(new Float(frameRate).intValue());

                        skipBytes(dataStream, 24);

                        int dwWidth = readUnsignedInt32(dataStream);
                        int dwHeight = readUnsignedInt32(dataStream);

                        setVideoResolution(dwWidth + "x" + dwHeight);
                    }

                    // Audio info
                    if (type == audi) {
                        streamCounter++;

                        skipBytes(dataStream, 4);
                        audioFccHandler = readUnsignedInt32(dataStream);

                        if (!audioCodecs.equals("")) {
                            audioCodecs += ", ";
                        }

                        FileInputStream fis = 
                            new FileInputStream("resources/FOURCCaudio.txt");
                        audioCodecs += findName(fis, "0x" + fromByteToAscii(audioFccHandler, 4));
                        skipBytes(dataStream, 12);
                        float sampleRate = readUnsignedInt32(dataStream);

                        audioSampleRate += (int) sampleRate;
                        skipBytes(dataStream, 16);

                        int channels = readUnsignedByte(dataStream);
                        audioChannels = channels;

                        skipBytes(dataStream, 3);

                        int bitRate = readUnsignedInt16(dataStream) / 1000;

                        audioBitRate = bitRate * 8;
                    }

                    if (type == vorb) {
                        streamCounter++;
                        if (!audioCodecs.equals("")) {
                            audioCodecs += ", ";
                        }
                        
                        audioCodecs += "Vorbis";
                        skipBytes(dataStream, 6);
                        audioChannels = readUnsignedByte(dataStream);
                        float sampleRate = readUnsignedInt16(dataStream);
                        audioSampleRate = (int) sampleRate;

                        skipBytes(dataStream, 6);

                        audioBitRate = readUnsignedInt32(dataStream) / 1000;
                    }

                    // Subtitles
                    if (type == text) {
                        streamCounter++;
                    }
                } else {
                    skipBytes(dataStream, 8);
                    type = readUnsignedByte(dataStream);
                    skipBytes(dataStream, 14);
                    int streamType = readUnsignedInt32(dataStream);

                    if (type != 0) {
                        // Subtitles 
                        if (streamType == vorb) {
                            String subtitles = getSubtitles();

                            if (!subtitles.equals("")) {
                                subtitles += ", ";
                            }

                            subtitles += getSubtitle(dataStream);
                            setSubtitles(subtitles);
                            streamCounter--;
                        } else if (streamType == ffff) { // Audio
                            streamCounter--;
                        }
                    } else {
                        if (streamCounter == 0) {
                            getExtendedCodecInfo(dataStream, 100);
                            quit = true;
                        }
                    }
                }
            }
        }

        setAudioCodec(audioCodecs);
        setAudioRate(audioSampleRate);
        setAudioBitRate(audioBitRate);
        setAudioChannels(audioChannels);
    }

    String getSubtitle(RandomAccessFile dataStream) {

        int temp;
        String subtitle = "";
        int counter = 0;

        try {

            while (counter++ < 100) {

                temp = readUnsignedByte(dataStream);

                if (temp == 0x4c && readUnsignedByte(dataStream) == 0x41
                                 && readUnsignedByte(dataStream) == 0x4e
                                 && readUnsignedByte(dataStream) == 0x47) {
                    skipBytes(dataStream, 5);

                    temp = readUnsignedByte(dataStream);

                    // 0x5b == [ 
                    while (temp != 0x01 && temp != 0x5b) {
                        subtitle += fromByteToAscii(temp, 1);
                        temp = readUnsignedByte(dataStream);
                    }
                    return subtitle;
                } else if (readUnsignedByte(dataStream) == 0x4f
                        && readUnsignedByte(dataStream) == 0x67
                        && readUnsignedByte(dataStream) == 0x67
                        && readUnsignedByte(dataStream) == 0x53) {

                    dataStream.seek(dataStream.getFilePointer() - 4);
                    return "";
                }
            }
        } catch (Exception e) {
            Messages.add(e);
        }
        return "";
    }

    /*
     * Calculates the size of the video only and then calculates the
     * videoBitRate/kbps.
     */
    private void calculateVideoBitRate(long fileSize) {
        int audioSize = 0;
        int duration = getDuration();
        int audioBitRate = 0;

        StringTokenizer string = new StringTokenizer("" + getAudioBitRate(), ", ");

        while (string.hasMoreTokens()) {
            // Audio Bit Rate in kbit/s
            audioBitRate = Integer.parseInt(string.nextToken()); 
            audioSize += ((audioBitRate * getDuration()) / 8) * 1000;
        }
        // Video rate kbit/s
        int videoBitRate = ((int) ((fileSize - audioSize) / getDuration()) / 1000) * 8; 
        setVideoBitRate(videoBitRate);
    }

    /**
     * Starts at the end of the file and finds the first OggS (Video) and
     * returns
     */
    private int getDuration(RandomAccessFile dataStream) {

        boolean found = false;
        int read;
        long length;
        long pointer;
        int duration = 0;

        try {
            /* Starts at end */
            length = dataStream.length() - 4;

            while (!found) {
                dataStream.seek(length--);
                read = readUnsignedInt32(dataStream);

                if (read == OggS) {
                    skipBytes(dataStream, 2);

                    duration = readUnsignedInt32(dataStream);
                    skipBytes(dataStream, 4);

                    if (readUnsignedByte(dataStream) == 0) /* 0 for video */
                        found = true;
                }
            }
        }

        catch (Exception e) {
            Messages.add(e);
            return 0;
        }

        return (int) ((float) duration / frameRate);
    }

    private void getExtendedCodecInfo(RandomAccessFile dataStream, int limit)
            throws Exception {

        int temp;
        String extendedInfo = "";

        while (limit-- > 0) {
            temp = readUnsignedByte(dataStream);

            /*44 == D, 58 == X*/
            if (Integer.toHexString(temp).equals("44")
                    || Integer.toHexString(temp).equals("58")) {
                extendedInfo = "";
                extendedInfo += fromByteToAscii(temp, 1);

                for (int u = 0; u < 3; u++) {
                    temp = readUnsignedByte(dataStream);
                    extendedInfo += fromByteToAscii(temp, 1);
                }

                if ((extendedInfo.toLowerCase().equals("divx"))
                        || (extendedInfo.toLowerCase().equals("xvid"))) {

                    for (int a = 0; a < 100; a++) {
                        temp = readUnsignedByte(dataStream);

                        if (temp == 0) {
                            break;
                        }
                        extendedInfo += fromByteToAscii(temp, 1);
                    }

                    /*If last character is not a digit it is removed*/
                    for (int i = 0; i < extendedInfo.length(); i++) {

                        if (!Character.isDigit(extendedInfo.charAt(extendedInfo
                                .length() - 1))) {
                            if (extendedInfo.charAt(extendedInfo.length() - 1) == 'p')
                                ;//Encoded with BVOP I think
                            extendedInfo = extendedInfo.substring(0,
                                    (extendedInfo.length() - 1));
                        } else
                            break;
                    }

                    /*Replaces "Build" with "b" if it occurs.*/
                    if ((extendedInfo.toLowerCase().startsWith("divx"))
                            && extendedInfo.length() > 12) {
                        if (extendedInfo.substring(7, 12).equals("Build")) {
                            extendedInfo = extendedInfo.replaceFirst("Build",
                                    "b");
                        }
                    }

                    FileInputStream fis = new FileInputStream(
                            "resources/videoExtended.txt");
                    String codecName = findName(fis, extendedInfo);
                    if (!codecName.equals("")) {
                        setVideoCodec(codecName);
                        return;
                    }
                }
            }
        }
    }
}
